@@ -4,6 +4,7 @@ module Termonad.Types where
 
 import Termonad.Prelude
 
+import Control.Monad.Fail (fail)
 import Data.FocusList (FocusList, emptyFL, singletonFL, getFocusItemFL, lengthFL)
 import Data.Unique (Unique, hashUnique, newUnique)
 import Data.Yaml
@@ -27,7 +28,7 @@ import GI.Gtk
 import GI.Pango (FontDescription)
 import GI.Vte (Terminal, CursorBlinkMode(..))
 import Text.Pretty.Simple (pPrint)
-import Text.Show (Show(showsPrec), ShowS, showParen, showString)
+import Text.Show (ShowS, showParen, showString)
 
 import Termonad.Gtk (widgetEq)
 
